@@ -4,9 +4,10 @@ from urllib.parse import parse_qs, urlencode, urlsplit, urlunsplit
 
 from codercore.lib.collection import Direction
 from fastapi import Request, Response
+from fastapi.params import Depends
 
 from coderfastapi.lib.signature import copy_parameters
-from coderfastapi.lib.validation.schemas.pagination import CursorSchema
+from coderfastapi.lib.validation.schemas.pagination import DeserializableCursor
 from coderfastapi.lib.validation.schemas.query import (
     OrderableQueryParameters,
     QueryParameters,
@@ -35,7 +36,10 @@ def paginate(
             **kwargs,
         ) -> Entities:
             schema_name, query_schema = _get_query_schema(func_signature, request)
-            kwargs[schema_name] = query_schema
+            if query_schema:
+                kwargs[schema_name] = query_schema
+            else:
+                query_schema = kwargs[schema_name]
 
             result = await func(*args, **kwargs)
             links = _build_links(id_attr, query_schema, request, result)
@@ -56,13 +60,24 @@ def paginate(
 def _get_query_schema(func_signature: Signature, request: Request) -> tuple[str, S]:
     for name, parameter in func_signature.parameters.items():
         if _is_valid_query_parameter(parameter):
-            return (name, parameter.annotation(**dict(request.query_params)))
+            if _is_injectable(parameter):
+                return (name, parameter.annotation(**dict(request.query_params)))
+            else:
+                return (name, None)
     raise KeyError("QuerySchema not found")
 
 
 def _is_valid_query_parameter(parameter: Parameter) -> bool:
-    return isclass(parameter.annotation) and issubclass(
-        parameter.annotation, QueryParameters
+    return (
+        isclass(parameter.annotation)
+        and getattr(parameter.annotation, "__canonical_name__", None)
+        == "QueryParameters"
+    )
+
+
+def _is_injectable(parameter) -> bool:
+    return (
+        isinstance(parameter.default, Depends) and parameter.default.dependency is None
     )
 
 
@@ -74,17 +89,20 @@ def _build_links(
 ) -> list[str]:
     result_length = len(result)
     links = []
-    value_attr = id_attr
-    if isinstance(query_schema, OrderableQueryParameters):
-        value_attr = query_schema.order_by
+    value_attr = getattr(query_schema, "order_by", id_attr)
 
-    if query_schema.cursor and result_length >= 1:
-        cursor = _create_cursor(Direction.DESC, id_attr, value_attr, result[0])
-        links.append(_construct_link(cursor, "previous", request))
-
-    if result_length == query_schema.limit:
-        cursor = _create_cursor(Direction.ASC, id_attr, value_attr, result[-1])
-        links.append(_construct_link(cursor, "next", request))
+    if previous_cursor := query_schema.cursor:
+        previous_direction = previous_cursor.direction
+        if previous_direction == Direction.ASC or result_length == query_schema.limit:
+            cursor = _create_cursor(Direction.DESC, id_attr, value_attr, result[0])
+            links.append(_construct_link(cursor, "previous", request))
+        if previous_direction == Direction.DESC or result_length == query_schema.limit:
+            cursor = _create_cursor(Direction.ASC, id_attr, value_attr, result[-1])
+            links.append(_construct_link(cursor, "next", request))
+    else:
+        if result_length == query_schema.limit:
+            cursor = _create_cursor(Direction.ASC, id_attr, value_attr, result[-1])
+            links.append(_construct_link(cursor, "next", request))
 
     return links
 
@@ -94,20 +112,20 @@ def _create_cursor(
     id_attr: str,
     value_attr: str,
     item: T,
-) -> CursorSchema:
-    return CursorSchema(
+) -> DeserializableCursor:
+    return DeserializableCursor(
         last_id=getattr(item, id_attr),
         last_value=getattr(item, value_attr),
         direction=direction,
     )
 
 
-def _construct_link(cursor: CursorSchema, rel: str, request: Request) -> str:
+def _construct_link(cursor: DeserializableCursor, rel: str, request: Request) -> str:
     url = _construct_url_with_cursor(cursor, request)
     return f'<{url}>; rel="{rel}"'
 
 
-def _construct_url_with_cursor(cursor: CursorSchema, request: Request) -> str:
+def _construct_url_with_cursor(cursor: DeserializableCursor, request: Request) -> str:
     scheme, netloc, path, query_string, fragment = urlsplit(str(request.url))
     query_params = parse_qs(query_string)
     query_params["cursor"] = [str(cursor)]
